import datetime
import os
import subprocess

from django.conf import settings
from django.core.management.base import BaseCommand, CommandError

import dropbox
from dropbox.exceptions import AuthError
from ii_django_backup.settings import backup_settings as bs


class Command(BaseCommand):
    help = 'Make backup of Django database'

    def check(self):
        """
        Check if ii-django-backup is configured properly.

        :return: None
        :raises CommandError: if ii-django-backup is not configured correctly
        """
        backup_dir = bs.DIR
        dropbox_dir = bs.DROPBOX_DIR
        dropbox_access_token = bs.DROPBOX_ACCESS_TOKEN
        use_dropbox = (dropbox_dir is not None) or (
            dropbox_access_token is not None)

        if backup_dir is None:
            raise CommandError(
                'Please declare II_DJANGO_BACKUP[\'DIR\'], as the default '
                'backup dir is based on settings.BASE_DIR, and your settings '
                'don\'t define a BASE_DIR.')

        if use_dropbox and (dropbox_dir is None):
            raise CommandError(
                'Please declare II_DJANGO_BACKUP[\'DROPBOX_DIR\'] or disable '
                'using dropbox by setting II_DJANGO_BACKUP[\'DROPBOX_ACCESS_'
                'TOKEN\'] to None')

        if use_dropbox and (dropbox_access_token is None):
            raise CommandError(
                'Please declare II_DJANGO_BACKUP[\'DROPBOX_ACCESS_TOKEN\'] or '
                'disable using dropbox by setting II_DJANGO_BACKUP[\'DROPBOX_D'
                'IR\'] to None')

    def get_backup_command(self, db_dict, backup_path):
        """
        Return backup shell command for given database dict and path.

        :param dict db_dict: used database information
        :param str backup_path: used path to export backup to

        :return: the backup command string
        :rtype: str
        :raises CommandError: if the db_dict['ENGINE'] is unknown
        """
        db_type = db_dict['ENGINE']

        if db_type in ['django.db.backends.postgresql_psycopg2',
                       'django.contrib.gis.db.backends.postgis']:
            cmd = ('pg_dump -U {USER} -h {HOST} -p {PORT} -Fc {NAME}'
                   ' --no-owner').format(**db_dict)
        elif db_type == 'django.db.backends.mysql':
            cmd = 'mysqldump --opt -Q -u {USER} {NAME}'.format(**db_dict)
        else:
            raise CommandError(
                'No backup command is implemented for {}'.format(db_type))

        if bs.USE_GZIP:
            return '{cdm} | gzip > {backup_path}'.format(
                cdm=cmd, backup_path=backup_path)

        return '{cdm} > {backup_path}'.format(cdm=cmd, backup_path=backup_path)

    def handle(self, *args, **options):
        """
        Creates backup for Django's default database connection.
        If configured, it will also upload this backup to the given dropbox
        folder.

        :return: None
        :raises CommandError: if ii-django-backup is not configured correctly
        :raises CommandError: if the db_dict['ENGINE'] is unknown
        :raises CommandError: if script couldn't connect to Dropbox
        """
        self.check()

        # gather configuration variables
        db_dict = settings.DATABASES['default']

        # create an unique backup filename
        filename = bs.NAME_GENERATOR_FUNC(db_dict=db_dict,
                                          dt=datetime.datetime.now())

        # when using GZIP, add .gz extension to the filename
        if bs.USE_GZIP:
            filename = '{}.gz'.format(filename)

        backup_dir = bs.DIR
        backup_path = os.path.join(backup_dir, filename)

        # create backup-dir if it doesn't exists yet
        if not os.path.isdir(backup_dir):
            os.mkdir(backup_dir)

        # backup locally
        cmd = self.get_backup_command(db_dict, backup_path)
        subprocess.call(cmd, shell=True)

        # backup to dropbox if needed
        if bs.DROPBOX_ACCESS_TOKEN is not None:
            try:
                dbx = dropbox.Dropbox(bs.DROPBOX_ACCESS_TOKEN)
                with open(backup_path, 'rb') as f:
                    data = f.read()
                    dbx.files_upload(data, '{}{}'.format(
                        bs.DROPBOX_DIR, filename))
            except AuthError:
                raise CommandError(
                    'Could not connect to Dropbox. Please check II_DJANGO_BAC'
                    'KUP[\'DROPBOX_TOKEN\'].')



def default(db_dict, dt):
    db_name = db_dict['NAME']
    engine = db_dict['ENGINE']

    if engine in ['django.db.backends.postgresql_psycopg2',
                  'django.contrib.gis.db.backends.postgis']:
        extension = '.backup'
    else:
        extension = '.sql'

    return '{time}_db_backup-{db_name}{extension}'.format(
        time=dt.strftime('%Y.%m.%d.%H.%M.%S'),
        db_name=db_name,
        extension=extension
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import re
from io import open

from setuptools import setup


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


def get_packages(package):
    """
    Return root package and all sub-packages.
    """
    return [dirpath
            for dirpath, dirnames, filenames in os.walk(package)
            if os.path.exists(os.path.join(dirpath, '__init__.py'))]


def get_package_data(package):
    """
    Return all files under the root package, that are not in a
    package themselves.
    """
    walk = [(dirpath.replace(package + os.sep, '', 1), filenames)
            for dirpath, dirnames, filenames in os.walk(package)
            if not os.path.exists(os.path.join(dirpath, '__init__.py'))]

    filepaths = []
    for base, filenames in walk:
        filepaths.extend([os.path.join(base, filename)
                          for filename in filenames])

    return {package: filepaths}


version = get_version('ii_django_backup')

setup(
    name='ii_django_backup',
    version=version,
    description='ideallical django backup',
    url='https://github.com/ideallical/ii-django-backup',
    download_url=('https://github.com/ideallical/ii-django-backup/archive/'
                  '0.1.tar.gz'),
    author='ideallical',
    author_email='info@ideallical.com',
    keywords=['django', 'backup', 'dropbox'],
    license='BSD',
    install_requires=[
        'dropbox>=7.1.1',
    ],
    packages=get_packages('ii_django_backup'),
    package_data=get_package_data('ii_django_backup'),
    zip_safe=False
)



"""
Settings for ideallical drf pagination are all namespaced in the
II_DJANGO_BACKUP setting.
For example your project's `settings.py` file might look like this:

II_DJANGO_BACKUP = {
    'DIR': '/var/backups/',
    'DROPBOX_ACCESS_TOKEN': '**********',
    'DROPBOX_DIR': '/websitename/staging/',
    'USE_GZIP': True,
}

This module provides the `backup_setting` object, that is used to access
ideallical Django backup settings, checking for user settings first, then
falling back to the defaults.
"""
from __future__ import unicode_literals

import os
from importlib import import_module

from django.conf import settings
from django.test.signals import setting_changed
from django.utils import six

try:
    backup_dir = os.path.join(settings.BASE_DIR, 'backups')
except AttributeError:
    backup_dir = None


DEFAULTS = {
    'DIR': backup_dir,
    'NAME_GENERATOR_FUNC': 'ii_django_backup.name_generators.default',
    'DROPBOX_ACCESS_TOKEN': None,
    'DROPBOX_DIR': None,
    'USE_GZIP': True,
}


# List of settings that may be in string import notation.
IMPORT_STRINGS = (
    'NAME_GENERATOR_FUNC',
)


# List of settings that have been removed
REMOVED_SETTINGS = ()


def perform_import(val, setting_name):
    """
    If the given setting is a string import notation,
    then perform the necessary import or imports.
    """
    if val is None:
        return None
    elif isinstance(val, six.string_types):
        return import_from_string(val, setting_name)
    elif isinstance(val, (list, tuple)):
        return [import_from_string(item, setting_name) for item in val]
    return val


def import_from_string(val, setting_name):
    """
    Attempt to import a class from a string representation.
    """
    try:
        # Nod to tastypie's use of importlib.
        parts = val.split('.')
        module_path, class_name = '.'.join(parts[:-1]), parts[-1]
        module = import_module(module_path)
        return getattr(module, class_name)
    except (ImportError, AttributeError) as e:
        msg = "Could not import '{}' for backup setting '{}'. {}: {}.".format(
            val, setting_name, e.__class__.__name__, e)
        raise ImportError(msg)


class BackupSettings(object):
    """
    A settings object, that allows Backup settings to be accessed as
    properties.
    For example:

        from ii_django_backup.settings import backup_settings
        print(backup_settings.DIR)

    Any setting with string import paths will be automatically resolved
    and return the class, rather than the string literal.
    """
    def __init__(self, user_settings=None, defaults=None, import_strings=None):
        if user_settings:
            self._user_settings = self.__check_user_settings(user_settings)
        self.defaults = defaults or DEFAULTS
        self.import_strings = import_strings or IMPORT_STRINGS

    @property
    def user_settings(self):
        if not hasattr(self, '_user_settings'):
            self._user_settings = getattr(settings, 'II_DJANGO_BACKUP', {})
        return self._user_settings

    def __getattr__(self, attr):
        if attr not in self.defaults:
            raise AttributeError("Invalid Backup setting: '%s'" % attr)

        try:
            # Check if present in user settings
            val = self.user_settings[attr]
        except KeyError:
            # Fall back to defaults
            val = self.defaults[attr]

        # Coerce import strings into classes
        if attr in self.import_strings:
            val = perform_import(val, attr)

        # Cache the result
        setattr(self, attr, val)
        return val

    def __check_user_settings(self, user_settings):
        SETTINGS_DOC = "https://github.com/ideallical/ii-django-backup/"
        for setting in REMOVED_SETTINGS:
            if setting in user_settings:
                raise RuntimeError(
                    "The '{}' setting has been removed. Please refer to '{}' "
                    "for available settings.".format(setting, SETTINGS_DOC))
        return user_settings


backup_settings = BackupSettings(None, DEFAULTS, IMPORT_STRINGS)


def reload_backup_settings(*args, **kwargs):
    global backup_settings
    setting, value = kwargs['setting'], kwargs['value']
    if setting == 'II_DJANGO_BACKUP':
        backup_settings = BackupSettings(
            value, DEFAULTS, IMPORT_STRINGS)


setting_changed.connect(reload_backup_settings)
